// main.c from CH552oneClickCompiler
// RR20210807 002  change flash rate and pins
//            003  CHANGE NUMBERING TO 10-17 & 30-35 
// RR20210808 004  remove debug.h / debug.c --> misc.h / misc.c & compile004.bat
//            005  simplify to flash only P1.4; include routines from debug.c (misc.c)
//                 works for CH554 but not CH552oneClickCompiler
//            006  simplify
// RR20210809 007  try Thomas simpler compile.bat
//            008  include in this file stdint.h ch554.h compile.h (and previously debug.c)
//            009  simplified and self-contained
//            010a remove more



// ============== copied from files ====================================================================================
// Debug.C		* Author: WCH	V1.0	2017/01/20	(c)
// stdint.h 	- ISO C99 7.18 Integer types <stdint.h>	(C) 2005, Maarten Brock, sourceforge.brock@dse.nl	(C) 2011, Philipp Klaus Krause, pkk@spth.de
// CH554.h 		Copyright  (C)  W.ch  1999-2014
// compiler.h	Copyright (C) 2006, Maarten Brock, sourceforge.brock@dse.nl	Portions of this file are Copyright 2014 Silicon Laboratories, Inc.
// =====================================================================================================================

// ===== stdint ========================================================================================================

/* Signed.  */
typedef signed char             int8_t;
typedef short int               int16_t;
typedef long int                int32_t;
#ifdef __SDCC_LONGLONG
typedef long long int           int64_t;
#endif

/* Unsigned.  */
typedef unsigned char           uint8_t;
typedef unsigned short int      uint16_t;
typedef unsigned long int       uint32_t;
#ifdef __SDCC_LONGLONG
typedef unsigned long long int  uint64_t;
#endif

// =====================================================================================================================

// ===== compiler.h ====================================================================================================
/** SDCC - Small Device C Compiler
  * http://sdcc.sf.net
 */
#if defined (SDCC) || defined (__SDCC)
# define SBIT(name, addr, bit)  __sbit  __at(addr+bit)                    name
# define SFR(name, addr)        __sfr   __at(addr)                        name
# define SFRX(name, addr)       __xdata volatile unsigned char __at(addr) name
# define SFR16(name, addr)      __sfr16 __at(((addr+1U)<<8) | addr)       name
# define SFR16E(name, fulladdr) __sfr16 __at(fulladdr)                    name
# define SFR16LEX(name, addr)   __xdata volatile unsigned short __at(addr) name
# define SFR32(name, addr)      __sfr32 __at(((addr+3UL)<<24) | ((addr+2UL)<<16) | ((addr+1UL)<<8) | addr) name
# define SFR32E(name, fulladdr) __sfr32 __at(fulladdr)                    name

# define INTERRUPT(name, vector) void name (void) __interrupt (vector)
# define INTERRUPT_USING(name, vector, regnum) void name (void) __interrupt (vector) __using (regnum)

// NOP () macro support
#define NOP() __asm NOP __endasm

/** default
  * unrecognized compiler
 */
#else
# warning unrecognized compiler

#endif

// =====================================================================================================================

// ===== CH554.h =======================================================================================================
/*----- SFR --------------------------------------------------------------*/
/*  sbit are bit addressable, others are byte addressable */
/*  System Registers  */
SFR(PSW,	0xD0);	// program status word
#define MASK_PSW_RS       0x18      // bit mask of register R0-R7 bank selection
SFR(ACC,	0xE0);	// accumulator
SFR(B,	0xF0);	// general purpose register B
SFR(SP,	0x81);	// stack pointer
//sfr16 DPTR          = 0x82;         // DPTR pointer, little-endian
SFR(DPL,	0x82);	// data pointer low
SFR(DPH,	0x83);	// data pointer high
SFR(SAFE_MOD,	0xA1);	// WriteOnly: writing safe mode
//sfr CHIP_ID         = 0xA1;         // ReadOnly: reading chip ID
// ??? #define CHIP_ID           SAFE_MOD
SFR(GLOBAL_CFG,	0xB1);	// global config, Write@SafeMode
SFR(PCON,	0x87);	// power control and reset flag
SFR(CLOCK_CFG,	0xB9);	// system clock config: lower 3 bits for system clock Fsys, Write@SafeMode
#define bOSC_EN_INT       0x80      // internal oscillator enable and original clock selection: 1=enable & select internal clock, 0=disable & select external clock
#define bWDOG_IF_TO       0x20      // ReadOnly: watch-dog timer overflow interrupt flag, cleared by reload watch-dog count or auto cleared when MCU enter interrupt routine
#define bROM_CLK_FAST     0x10      // flash-ROM clock frequency selection: 0=normal(for Fosc>=16MHz), 1=fast(for Fosc<16MHz)
#define MASK_SYS_CK_SEL   0x07      // bit mask of system clock Fsys selection

/*  Port Registers  */
SFR(P1,	0x90);	// port 1 input & output
SFR(P1_MOD_OC,	0x92);	// port 1 output mode: 0=push-pull, 1=open-drain
SFR(P1_DIR_PU,	0x93);	// port 1 direction for push-pull or pullup enable for open-drain
SFR(P3,	0xB0);	// port 3 input & output
SFR(P3_MOD_OC,	0x96);	// port 3 output mode: 0=push-pull, 1=open-drain
SFR(P3_DIR_PU,	0x97);	// port 3 direction for push-pull or pullup enable for open-drain
SFR(PIN_FUNC,	0xC6);	// pin function selection
SFR(XBUS_AUX,	0xA2);	// xBUS auxiliary setting

/*----- Reference Information --------------------------------------------*/
// #define ID_CH551          0x51      // chip ID	RR20210808
// #define ID_CH552          0x52      // chip ID  	RR20210808
// #define ID_CH554          0x54      // chip ID

// =====================================================================================================================

// ===== debug.h =======================================================================================================
#ifndef FREQ_SYS
#define	FREQ_SYS	24000000
#endif

void	CfgFsys( )  
{
	SAFE_MOD = 0x55;
	SAFE_MOD = 0xAA;
//     CLOCK_CFG |= bOSC_EN_XT;                          //使能外部晶振
//     CLOCK_CFG &= ~bOSC_EN_INT;                        //关闭内部晶振    

#if FREQ_SYS == 24000000
	CLOCK_CFG = CLOCK_CFG & ~ MASK_SYS_CK_SEL | 0x06;  // 24MHz	
#else
	#warning FREQ_SYS invalid or not set
#endif

	SAFE_MOD = 0x00;
}

void	mDelayuS( uint16_t n )  // 以uS为单位延时
{
	while ( n ) {  // total = 12~13 Fsys cycles, 1uS @Fsys=12MHz
// set for 24MHz...
		++ SAFE_MOD;  // 2 Fsys cycles, for higher Fsys, add operation here
		++ SAFE_MOD;
		++ SAFE_MOD;
		++ SAFE_MOD;
		++ SAFE_MOD;
		++ SAFE_MOD;
		++ SAFE_MOD;
		-- n;
	}
}

void	mDelaymS( uint16_t n )                                                  // 以mS为单位延时
{
	while ( n ) {
#ifdef	DELAY_MS_HW
		while ( ( TKEY_CTRL & bTKC_IF ) == 0 );
		while ( TKEY_CTRL & bTKC_IF );
#else
		mDelayuS( 1000 );
#endif
		-- n;
	}
}                                         

// =====================================================================================================================

// ===== main.c ========================================================================================================

uint16_t wait = 500;

// Port 1.4
#define LED_PIN14 4
SBIT(LED14, 0x90, LED_PIN14);
// Port 3.4
#define LED_PIN34 4
SBIT(LED34, 0xB0, LED_PIN34);


void main() {
    CfgFsys();

    P1_MOD_OC = P1_MOD_OC |(1<<LED_PIN14);
    P1_DIR_PU = P1_DIR_PU |	(1<<LED_PIN14);
	
    P3_MOD_OC = P3_MOD_OC |(1<<LED_PIN34);
    P3_DIR_PU = P3_DIR_PU |	(1<<LED_PIN34);
	
    while (1) {
    	mDelaymS(wait);
        LED14 = !LED14;
        LED34 = !LED34;
		}
}
